@@ -199,26 +199,23 @@ main (int argc, char *argv[])
   int res;
   char input[MAX_LINE_LENGTH];
   char *in;
-  struct sockaddr connect_addr;
 
-#ifdef HAVE_LOCALE_H
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
-#endif
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
   if (cmdline_parser (argc, argv, &args_info) != 0)
     return 1;
 
-  if (!args_info.client_flag &&
-      !args_info.server_flag &&
+  if (!args_info.client_flag && !args_info.server_flag &&
       !args_info.client_mechanisms_flag && !args_info.server_mechanisms_flag)
-    {
-      fprintf (stderr, "%s: missing argument\n", argv[0]);
-      cmdline_parser_print_help ();
-      printf ("\nReport bugs to <%s>\n", PACKAGE_BUGREPORT);
-      return 1;
-    }
+    error (EXIT_FAILURE, 0,
+	   "missing argument\nTry `%s --help' for more information.",
+	   program_name);
+
+  if (args_info.smtp_flag && args_info.imap_flag)
+    args_info.imap_flag = 0;
 
   if (args_info.imap_flag && !args_info.service_given)
     {
@@ -232,77 +229,111 @@ main (int argc, char *argv[])
       args_info.no_client_first_flag = 1;
     }
 
-  if (args_info.connect_given)
+  if (args_info.connect_given || args_info.inputs_num > 0)
     {
-      struct servent *se;
-      struct hostent *he;
-      struct sockaddr_in *sinaddr_inp = (struct sockaddr_in *) &connect_addr;
       char *connect_hostname;
       char *connect_service;
+      struct sockaddr connect_addr;
+      struct sockaddr *saddr = &connect_addr;
+      size_t saddrlen = sizeof (*saddr);
+#if HAVE_GETADDRINFO
+      struct addrinfo hints;
+      struct addrinfo *ai;
+#else
+      struct servent *se;
+      struct hostent *he;
+#endif
 
-      if (strrchr (args_info.connect_arg, ':'))
+      if (args_info.connect_given)
 	{
-	  connect_hostname = strdup (args_info.connect_arg);
-	  *strrchr (connect_hostname, ':') = '\0';
-	  connect_service = strdup (strrchr (args_info.connect_arg, ':') + 1);
+	  if (strrchr (args_info.connect_arg, ':'))
+	    {
+	      connect_hostname = strdup (args_info.connect_arg);
+	      *strrchr (connect_hostname, ':') = '\0';
+	      connect_service =
+		strdup (strrchr (args_info.connect_arg, ':') + 1);
+	    }
+	  else
+	    {
+	      connect_hostname = strdup (args_info.connect_arg);
+	      if (args_info.smtp_flag)
+		connect_service = strdup ("25");
+	      else
+		connect_service = strdup ("143");
+	    }
 	}
-      else
+      else if (args_info.inputs_num > 0)
 	{
-	  connect_hostname = strdup (args_info.connect_arg);
-	  if (args_info.smtp_flag)
+	  connect_hostname = args_info.inputs[0];
+	  if (args_info.inputs_num > 1)
+	    connect_service = args_info.inputs[1];
+	  else if (args_info.smtp_flag)
 	    connect_service = strdup ("25");
 	  else
 	    connect_service = strdup ("143");
 	}
 
-      he = gethostbyname (connect_hostname);
-      if (!he || he->h_addr_list[0] == NULL || he->h_addrtype != AF_INET)
-	{
-	  fprintf (stderr, "%s: unknown host: %s\n", argv[0],
-		   connect_hostname);
-	  return 1;
-	}
+#if HAVE_GETADDRINFO
+      memset (&hints, 0, sizeof (hints));
+      hints.ai_socktype = SOCK_STREAM;
+      res = getaddrinfo (connect_hostname, connect_service, &hints, &ai);
+      if (res != 0)
+	error (EXIT_FAILURE, 0, "%s: %s", connect_hostname,
+	       gai_strerror (res));
+      saddr = ai->ai_addr;
+      saddrlen = ai->ai_addrlen;
+#else
       memset (&connect_addr, 0, sizeof (connect_addr));
-      sinaddr_inp->sin_family = he->h_addrtype;
-      memcpy (&sinaddr_inp->sin_addr, he->h_addr_list[0], he->h_length);
-      se = getservbyname (connect_service, "tcp");
-      if (se)
-	sinaddr_inp->sin_port = se->s_port;
+      he = gethostbyname (connect_hostname);
+      if (!he || he->h_addr_list[0] == NULL)
+	error (EXIT_FAILURE, 0, "%s: %s", connect_hostname,
+	       hstrerror (h_errno));
+      saddr->sa_family = he->h_addrtype;
+      if (he->h_addrtype == AF_INET)
+	{
+	  struct sockaddr_in *saddrin = (struct sockaddr_in *) &connect_addr;
+	  memcpy (&saddrin->sin_addr, he->h_addr_list[0], he->h_length);
+	  se = getservbyname (connect_service, "tcp");
+	  if (se)
+	    saddrin->sin_port = se->s_port;
+	  else if (atoi (connect_service) == 0)
+	    error (EXIT_FAILURE, 0, "unknown service: %s", connect_service);
+	  else
+	    saddrin->sin_port = htons (atoi (connect_service));
+	}
       else
-	sinaddr_inp->sin_port = htons (atoi (connect_service));
-      if (sinaddr_inp->sin_port == 0 || sinaddr_inp->sin_port == htons (0))
-	{
-	  fprintf (stderr, "%s: unknown service: %s\n", argv[0],
-		   connect_service);
-	  return 1;
-	}
+	error (EXIT_FAILURE, 0, "unsupported address type: %d",
+	       he->h_addrtype);
+#endif
 
-      sockfd = socket (AF_INET, SOCK_STREAM, 0);
+      sockfd = socket (saddr->sa_family, SOCK_STREAM, 0);
       if (sockfd < 0)
+	error (EXIT_FAILURE, errno, "socket");
+
+      if (connect (sockfd, saddr, saddrlen) < 0)
 	{
-	  fprintf (stderr, "%s: ", argv[0]);
-	  perror ("socket");
-	  return 1;
+	  int save_errno = errno;
+	  close (sockfd);
+	  error (EXIT_FAILURE, save_errno, "connect");
 	}
 
-      if (connect (sockfd, &connect_addr, sizeof (connect_addr)) < 0)
-	{
-	  fprintf (stderr, "%s: ", argv[0]);
-	  perror ("connect");
-	  close (sockfd);
-	  return 1;
-	}
+#if HAVE_GETADDRINFO
+      freeaddrinfo (ai);
+#endif
+
       if (!args_info.hostname_arg)
 	args_info.hostname_arg = strdup (connect_hostname);
     }
+  else
+    {
+      cmdline_parser_print_help ();
+      return EXIT_SUCCESS;
+    }
 
   res = gsasl_init (&ctx);
   if (res != GSASL_OK)
-    {
-      fprintf (stderr, _("Libgsasl error (%d): %s\n"), res,
-	       gsasl_strerror (res));
-      return 1;
-    }
+    error (EXIT_FAILURE, 0, _("Libgsasl error (%d): %s"), res,
+	   gsasl_strerror (res));
 
   /* Set callbacks */
   if (args_info.maxbuf_arg != 0)
@@ -343,11 +374,8 @@ main (int argc, char *argv[])
 	res = gsasl_server_mechlist (ctx, &mechs);
 
       if (res != GSASL_OK)
-	{
-	  fprintf (stderr, _("Libgsasl error (%d): %s\n"), res,
-		   gsasl_strerror (res));
-	  return 1;
-	}
+	error (EXIT_FAILURE, 0, _("Libgsasl error (%d): %s"), res,
+	       gsasl_strerror (res));
 
       if (!args_info.quiet_given)
 	{
@@ -397,11 +425,8 @@ main (int argc, char *argv[])
       else
 	res = gsasl_server_start (ctx, mech, &xctx);
       if (res != GSASL_OK)
-	{
-	  fprintf (stderr, _("Libgsasl error (%d): %s\n"),
-		   res, gsasl_strerror (res));
-	  return 1;
-	}
+	error (EXIT_FAILURE, 0, _("Libgsasl error (%d): %s"), res,
+	       gsasl_strerror (res));
 
       in = NULL;
       out = NULL;
@@ -442,11 +467,8 @@ main (int argc, char *argv[])
       while (res == GSASL_NEEDS_MORE);
 
       if (res != GSASL_OK)
-	{
-	  fprintf (stderr, _("Libgsasl error (%d): %s\n"),
-		   res, gsasl_strerror (res));
-	  return 1;
-	}
+	error (EXIT_FAILURE, 0, _("Libgsasl error (%d): %s"), res,
+	       gsasl_strerror (res));
 
       if (!auth_finish ())
 	return 1;
@@ -537,11 +559,8 @@ main (int argc, char *argv[])
 	    }
 
 	  if (res != GSASL_OK)
-	    {
-	      fprintf (stderr, _("Libgsasl error (%d): %s\n"),
-		       res, gsasl_strerror (res));
-	      return 1;
-	    }
+	    error (EXIT_FAILURE, 0, _("Libgsasl error (%d): %s"), res,
+		   gsasl_strerror (res));
 	}
 
       if (!args_info.quiet_given)
