@@ -68,7 +68,7 @@
 #include "strdup.h"
 #include "progname.h"
 #include "error.h"
-#include "getpassword.h"
+#include "getpass.h"
 
 /* Get i18n. */
 #include <gettext.h>
